from grasping_type_inference.inference import Inference


def test_should_return_the_most_probable_result():
    inference = Inference(':LEFT-SIDE', ':BOTTOM', 'cup.n.01')
    result = inference.get_most_probable_result()

    assert 'LEFT-SIDE' == result

    inference = Inference(':LEFT-SIDE', ':BOTTOM', 'bowl.n.01')
    result = inference.get_most_probable_result()

    assert 'TOP' == result

    inference = Inference(':LEFT-SIDE', ':BOTTOM', 'spoon.n.01')
    result = inference.get_most_probable_result()

    assert 'TOP' == result


def test_should_return_probability_distribution():
    inference = Inference(':LEFT-SIDE', ':BOTTOM', 'cup.n.01')
    result = inference.get_storted_list_of_grasping_types_based_on_probability()

    assert list == type(result)

    inference = Inference(':LEFT-SIDE', ':BOTTOM', 'bowl.n.01')
    result = inference.get_storted_list_of_grasping_types_based_on_probability()

    assert list == type(result)

    inference = Inference(':LEFT-SIDE', ':BOTTOM', 'spoon.n.01')
    result = inference.get_storted_list_of_grasping_types_based_on_probability()

    assert list == type(result)


# Copyright 2018 Institute for Artificial Intelligence - University of Bremen
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
# documentation files (the "Software"), to deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of
# the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO
# THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
import os
from grasping_type_inference.definitions import ROOT_DIR
from high_level_markov_logic_network.markov_logic_network import MarkovLogicNetwork

MLN_PATH = os.path.join(ROOT_DIR, 'mln')


def train_all_grasping_mlns(path_to_training_files, path_to_mln_files=MLN_PATH):
    for training_file in os.listdir(path_to_training_files):
        if training_file.endswith('.db'):
            path_mln_training_file = os.path.join(path_to_training_files, training_file)
            mln_training_file_content = None

            with open(path_mln_training_file) as mln_training_file:
                mln_training_file_content = mln_training_file.read()

            #BOWL.train.db
            object_type = training_file.split('.')[0].lower()
            grasping_object_type_mln_name = 'grasping_{}.pracmln'.format(object_type)
            path_to_grasping_type_mln = os.path.join(path_to_mln_files, grasping_object_type_mln_name)
            grasping_object_type_mln = MarkovLogicNetwork(path_to_grasping_type_mln)
            print 'Learning {} ...'.format(grasping_object_type_mln_name)
            grasping_object_type_mln.learn(mln_training_file_content)
            print 'Learning is done'
            print ''




# -*- coding: utf-8 -*-

# Copyright © 2018 Institute for Artificial Intelligence - University of Bremen
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
# documentation files (the "Software"), to deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of
# the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO
# THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from grasping_type_inference.face import Face
from high_level_markov_logic_network.ground_atom import GroundAtom

__FACING_ROBOT_FACE__ = 'facing_robot_face'
__BOTTOM_FACE__ = 'bottom_face'
__OBJ_TO_BE_GRASPED__ = 'obj_to_be_grasped'
__IS_ROTATIONALLY_SYMMETRIC__ = 'is_rotationally_symmetric'
__GRASP_TYPE__ = 'grasp_type'


def get_facing_robot_face(arg):
    ground_atom_value = arg

    if type(arg) is Face:
        ground_atom_value = arg.value

    return GroundAtom(__FACING_ROBOT_FACE__, [ground_atom_value])


def get_bottom_face(arg):
    ground_atom_value = arg

    if type(arg) is Face:
        ground_atom_value = arg.value

    return GroundAtom(__BOTTOM_FACE__, [ground_atom_value])


def get_obj_to_be_grasped(arg):
    return GroundAtom(__OBJ_TO_BE_GRASPED__, [arg])


def get_is_rotationally_symmetric(arg):
    return GroundAtom(__IS_ROTATIONALLY_SYMMETRIC__, [arg])


def get_grasp_type(arg):
    return GroundAtom(__GRASP_TYPE__, [arg])


# -*- coding: utf-8 -*-

# Copyright © 2018 Institute for Artificial Intelligence - University of Bremen
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
# documentation files (the "Software"), to deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of
# the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO
# THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import os

ROOT_DIR = os.path.dirname(os.path.abspath(__file__))


# -*- coding: utf-8 -*-

# Copyright © 2018 Institute for Artificial Intelligence - University of Bremen
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
# documentation files (the "Software"), to deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of
# the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO
# THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from high_level_markov_logic_network.database import Database
from high_level_markov_logic_network.fuzzy_markov_logic_network.is_a_generator import get_is_a_ground_atoms
import grasping_type_inference.ground_atom_builder as gab
from grasping_type_inference.grasping_type_mln import load_grasping_mln, grasping_mln_selector_mln


class GraspingObject(object):
    def __init__(self, object_type, orientation):
        self.type = object_type
        self.orientation = orientation

    def _infer_required_grasping_mln_(self):
        evidence_database = self._create_evidence_database_for_grasping_selector_mln_()
        result = grasping_mln_selector_mln.infer(evidence_database)
        most_probable_result = _get_the_most_probable_result(result)

        return _remove_predicate_(most_probable_result)

    def get_grasping_types_probability_distribution(self):
        required_grasping_mln_name = self._infer_required_grasping_mln_()
        required_grasping_mln = '{}.pracmln'.format(required_grasping_mln_name)
        grasping_mln = load_grasping_mln(required_grasping_mln)

        evidence_database = self._create_evidence_database_for_grasping_mln_(grasping_mln)

        return grasping_mln.infer(evidence_database)

    def _create_evidence_database_for_grasping_selector_mln_(self):
        return self._create_grasping_object_related_evidences_with_given_mln_(grasping_mln_selector_mln)

    def _create_evidence_database_for_grasping_mln_(self, grasping_mln):
        evidence_database = self._create_grasping_object_related_evidences_with_given_mln_(grasping_mln)
        evidence_database.add_ground_atom(self.orientation.transform_facing_robot_face_to_ground_atom())
        evidence_database.add_ground_atom(self.orientation.transform_bottom_face_to_ground_atom())

        return evidence_database

    def _create_grasping_object_related_evidences_with_given_mln_(self, mln):
        evidence_database = Database(mln)
        evidence_database.add_ground_atom(gab.get_obj_to_be_grasped(self.type))

        learned_objects = mln.domains['learnedObject']
        is_a_ground_atoms = get_is_a_ground_atoms(self.type, learned_objects)
        for ground_atom in is_a_ground_atoms:
            evidence_database.add_ground_atom(ground_atom)

        return evidence_database

    def __eq__(self, other):
        if (self.type != other.type) or (self.orientation != other.orientation):
            return False
        else:
            return True


def _get_the_most_probable_result(result):
    solution = None
    max_prob = 0

    for ground_atom in result.results.keys():
        if result.results[ground_atom] >= max_prob:
            solution = ground_atom
            max_prob = result.results[ground_atom]

    return solution


def _remove_predicate_(atom):
    splited_atom = atom.split('(')[1]

    return splited_atom.split(')')[0]



# -*- coding: utf-8 -*-

# Copyright © 2018 Institute for Artificial Intelligence - University of Bremen
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
# documentation files (the "Software"), to deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of
# the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO
# THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="grasping_type_inference",
    version="1.0.0",
    author="Sebastian Koralewski",
    author_email="seba@cs.uni-bremen.de",
    description="Utilizes a Markov Logic Network to infer the most probable grasping type for a defined object",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/code-iai/iai-grasping-type-inference",
    packages=setuptools.find_packages(exclude=("test",)),
    install_requires=['high_level_markov_logic_network==1.0.1'],
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 2",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)


# -*- coding: utf-8 -*-

# Copyright © 2018 Institute for Artificial Intelligence - University of Bremen
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
# documentation files (the "Software"), to deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of
# the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO
# THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from grasping_type_inference.grasping_object.grasping_object import GraspingObject
from grasping_type_inference.grasping_object.orientation import Orientation


class Inference(object):
    def __init__(self, *evidences):
        self._facing_robot_face, self._bottom_face, self._object_type = evidences

    def get_storted_list_of_grasping_types_based_on_probability(self):
        result = self._get_grasping_types_probability_distribution()
        unsorted_results = result.results
        sorted_results = sorted(unsorted_results.items(), key=lambda x: x[1], reverse=True)
        solutions = []

        for sorted_result in sorted_results:
            solutions.append(sorted_result[0])

        return map(_remove_grasping_type_predicate, solutions)

    def get_most_probable_result(self):
        result = self._get_grasping_types_probability_distribution()

        solution = None
        max_prob = 0

        for ground_atom in result.results.keys():
            if result.results[ground_atom] >= max_prob:
                solution = ground_atom
                max_prob = result.results[ground_atom]

        return _remove_grasping_type_predicate(solution)

    def _get_grasping_types_probability_distribution(self):
        orientation = Orientation(self._facing_robot_face, self._bottom_face)
        grasping_object = GraspingObject(self._object_type, orientation)

        return grasping_object.get_grasping_types_probability_distribution()


def _remove_grasping_type_predicate(atom):
    splited_atom = atom.split('(')[1]

    return splited_atom.split(')')[0]


# -*- coding: utf-8 -*-

# Copyright © 2018 Institute for Artificial Intelligence - University of Bremen
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
# documentation files (the "Software"), to deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of
# the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO
# THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import grasping_type_inference.ground_atom_builder as gab


class Orientation:
    def __init__(self, facing_robot_face, bottom_face):
        self.facing_robot_face = facing_robot_face
        self.bottom_face = bottom_face

    def transform_facing_robot_face_to_ground_atom(self):
        return gab.get_facing_robot_face(self.facing_robot_face)

    def transform_bottom_face_to_ground_atom(self):
        return gab.get_bottom_face(self.bottom_face)

    def __eq__(self, other):
        if (self.facing_robot_face != other.facing_robot_face) or (self.bottom_face != other.bottom_face):
            return False
        else:
            return True




import os

from grasping_type_inference.learning import train_all_grasping_mlns
from grasping_type_inference.definitions import ROOT_DIR


TRAINING_DATA_FILE_PATH = os.path.join(ROOT_DIR, 'mln')
MLN_DATA_FILE_PATH = os.path.join(ROOT_DIR, 'test', 'mln')


def test_should_train_all_mlns_successful():
    train_all_grasping_mlns(TRAINING_DATA_FILE_PATH)


def test_should_train_all_mlns_successful_with_parameter():
    train_all_grasping_mlns(TRAINING_DATA_FILE_PATH, MLN_DATA_FILE_PATH)